@@ -1419,6 +1419,10 @@ src_presentation_sro_get_pres_for_magnifier (SRObject *obj,
     gchar *role;
     SRCPresentationBoundsType btype;
     gchar *presentation_ = NULL;
+    gint default_int_val = 0;
+    
+    static gint size_x = 0,
+	        size_y = 0;
 
     sru_assert (obj && reason);
 
@@ -1432,8 +1436,29 @@ src_presentation_sro_get_pres_for_magnifier (SRObject *obj,
 	switch (btype)
 	{
 	    case  SRC_PRESENTATION_BOUNDS_TYPE_OBJECT:
-		    sro_get_location (obj, SR_COORD_TYPE_SCREEN, &location, SR_INDEX_CONTAINER);
-		    break;
+		if (size_x == 0)
+		{
+    		    srconf_get_data_with_default (MAGNIFIER_DISPLAY_SIZE_X, 
+		    		    	          CFGT_INT, 
+				    	    	  (gpointer)&size_x, 
+				    	    	  (gpointer)&default_int_val, 
+				    	    	  MAGNIFIER_CONFIG_PATH);
+		}
+
+		if (size_y == 0)
+		{
+		    srconf_get_data_with_default (MAGNIFIER_DISPLAY_SIZE_Y, 
+				    	          CFGT_INT, 
+				    	    	  (gpointer)&size_y, 
+				    	    	  (gpointer)&default_int_val, 
+				    	    	  MAGNIFIER_CONFIG_PATH);
+		}
+
+		sro_get_location (obj, SR_COORD_TYPE_SCREEN, &location, SR_INDEX_CONTAINER);
+		if ((location.width - location.x)  >= size_x + 1 &&
+    		    (location.height - location.y) >= size_y + 1)
+			get = FALSE;
+		break;
 	    case SRC_PRESENTATION_BOUNDS_TYPE_TEXT_CHAR:
 		    sro_text_get_caret_location (obj, SR_COORD_TYPE_SCREEN, 
 				&location, SR_INDEX_CONTAINER);
@@ -1448,6 +1473,7 @@ src_presentation_sro_get_pres_for_magnifier (SRObject *obj,
 		sru_assert_not_reached ();
 		break;	    
 	}
+		
 	if (get)
 	{
 	    presentation_ = g_strdup_printf ("ROILeft =\"%d\" ROITop =\"%d\" ROIWidth =\"%d\" ROIHeight=\"%d\"",
