@@ -195,6 +195,7 @@ static GQueue			*srl_event_queue	= NULL;
 static AccessibleEventListener 	*srl_event_listeners[G_N_ELEMENTS (srl_events_type_name)];
 static Accessible		*srl_last_focus		= NULL;
 static Accessible		*srl_last_focus2	= NULL;
+static Accessible		*srl_last_at_spi_focus	= NULL;
 static Accessible		*srl_watched_acc	= NULL;
 static Accessible 		*srl_last_table 	= NULL;
 static Accessible 		*srl_last_context 	= NULL;
@@ -985,11 +986,11 @@ srl_notify_clients_obj (SRLEvent *event,
 		    sre_release_reference (pev);
 		}
 	    }
-	    else if (srle_has_type (event, SRL_EVENT_FOCUS))
-		srl_set_last_table (NULL);
 	    if (pacc)
 		Accessible_unref (pacc);
 	}
+	else if (srle_has_type (event, SRL_EVENT_FOCUS))
+	    srl_set_last_table (NULL);
     }
 
     if (obj)
@@ -1248,11 +1249,12 @@ srle_is_for_focused_ancestor_acc (SRLEvent *event)
     srl_assert (event);
 
     acc = event->acc_ev->source;
+
     Accessible_ref (acc);
     while (acc)
     {
 	Accessible *tmp;
-	rv = srl_acc_has_state (acc, SPI_STATE_FOCUSED);
+	rv = acc == srl_last_at_spi_focus;
 	if (rv)
 	    break;
 	tmp = acc;
@@ -1763,14 +1765,13 @@ srl_selection_event_is_reported (SRLEvent *event)
 }
 
 
+static SRLLastInfo last_info_focus, last_info_watched;
 
 static gboolean
 srl_event_is_reported (SRLEvent *event)
 {
     gboolean rv = FALSE;
-    static SRLLastInfo last_info_focus, last_info_watched;
     SRLLastInfo *last_info = NULL;
-
     srl_assert (event);
     last_info = srle_is_for_focused_acc (event) ? &last_info_focus : &last_info_watched;
 
@@ -1811,12 +1812,16 @@ srl_event_is_reported (SRLEvent *event)
 	rv = TRUE;
     else if (srle_has_type (event, SRL_EVENT_FOCUS2))
 	rv = TRUE;
+    else if (srle_has_type (event, SRL_EVENT_STATE_EXPANDED))
+    {
+	rv = srle_is_for_focused_ancestor_acc (event);
+	if (rv)
+	    srl_set_last_focus ((Accessible*) srle_get_acc (event));
+    }
     else if (srle_is_for_focused_acc (event) ||
 	     srle_is_for_watched_acc (event))
     {
-	if (srle_has_type (event, SRL_EVENT_STATE_EXPANDED)) 
-	    rv = srle_acc_has_role (event, SPI_ROLE_TABLE_CELL);
-	else if (srle_has_type (event, SRL_EVENT_STATE_CHECKED))
+	if (srle_has_type (event, SRL_EVENT_STATE_CHECKED))
 	{
 	    if (srle_acc_has_role (event, SPI_ROLE_RADIO_BUTTON))
 		rv = !last_info->checked && srle_acc_has_state (event, SPI_STATE_CHECKED);
@@ -1930,6 +1935,18 @@ srl_event_is_reported (SRLEvent *event)
     return rv;
 }
 
+static gboolean
+srl_set_last_at_spi_focus (Accessible *acc)
+{
+    if (srl_last_at_spi_focus)
+	Accessible_unref (srl_last_at_spi_focus);
+    srl_last_at_spi_focus = acc;
+    if (srl_last_at_spi_focus)
+	Accessible_ref (srl_last_at_spi_focus);
+    return TRUE;
+}
+
+
 static gboolean
 srl_set_last_focus (Accessible *acc)
 {
@@ -1961,6 +1978,9 @@ srl_process_event (SRLEvent *event)
 
     srl_assert (event);
 
+    if (srle_has_type (event, SRL_EVENT_FOCUS))
+	srl_set_last_at_spi_focus ((Accessible*)srle_get_acc (event));
+
     srl_timing_process_start ((gpointer) event->acc_ev);
     srle_set_acc (event);
     srle_change_type (event);
@@ -2054,6 +2074,7 @@ srl_init ()
 	srl_last_events[i] = NULL;
     srl_last_focus	= NULL;
     srl_last_focus2    	= NULL;
+    srl_last_at_spi_focus= NULL;
     srl_last_edit	= NULL;
     srl_idle_need	= FALSE;
     srl_idle_installed	= FALSE;
@@ -2119,6 +2140,8 @@ srl_terminate ()
 	Accessible_unref (srl_last_focus);
     if (srl_last_focus2)
 	Accessible_unref (srl_last_focus2);	
+    if (srl_last_at_spi_focus)
+	Accessible_unref (srl_last_at_spi_focus);
     if (srl_last_edit)
         Accessible_unref (srl_last_edit);
     if (srl_watched_acc)
@@ -2275,3 +2298,115 @@ srl_is_object_focused (SRObject *obj)
 
     return sro_get_acc (obj) == srl_last_focus || sro_get_acc (obj) == srl_last_focus2;
 }
+
+static Accessible*
+srl_get_focused_from_acc (Accessible *acc)
+{
+    Accessible *focused = NULL;
+    gint cnt, i;
+
+    srl_assert (acc);
+    
+    cnt = Accessible_getChildCount (acc);
+    for (i = 0; i < cnt; i++)
+    {
+	Accessible *child;
+	child = Accessible_getChildAtIndex (acc, i);
+	if (!child)
+	    continue;
+	if (srl_acc_has_state (child, SPI_STATE_FOCUSED))
+	{
+	    focused = child;
+	    Accessible_ref (focused);
+	}
+	else
+	    focused = srl_get_focused_from_acc (child);
+	
+	Accessible_unref (child);
+	if (focused)
+	    break;
+    }
+
+    return focused;
+}
+
+
+static Accessible*
+srl_get_focused_from_active_toplevel (Accessible *window)
+{
+    return srl_get_focused_from_acc (window);
+}
+
+static Accessible*
+srl_get_active_toplevel_from_app (Accessible *app)
+{
+    Accessible *active = NULL;
+    gint cnt, i;
+
+    srl_assert (app);
+    
+    cnt = Accessible_getChildCount (app);
+    for (i = 0; i < cnt; i++)
+    {
+	Accessible *child;
+	child = Accessible_getChildAtIndex (app, i);
+	if (!child)
+	    continue;
+	if (srl_acc_has_state (child, SPI_STATE_ACTIVE))
+	{
+	    active = child;
+	    break;
+	}
+	Accessible_unref (child);
+    }
+
+    return active;
+}
+
+static Accessible*
+srl_get_focused_from_desktop ()
+{
+    Accessible *desktop, *focused = NULL;
+    gint cnt, i;
+
+    desktop = SPI_getDesktop (0);
+    if (!desktop)
+	return NULL;
+    cnt = Accessible_getChildCount (desktop);
+    for (i = 0; i < cnt; i++)
+    {
+	Accessible *child = NULL, *active = NULL;
+	child = Accessible_getChildAtIndex (desktop, i);
+	if (child)
+	    active = srl_get_active_toplevel_from_app (child);
+	if (active)
+	    focused = srl_get_focused_from_active_toplevel (active);
+	if (child)
+	    Accessible_unref (child);
+	if (active)
+	    Accessible_unref (active);
+	if (focused)
+	    break;
+    }
+    Accessible_unref (desktop);
+
+    return focused;
+}
+
+SRObject*
+srl_get_focus_object ()
+{
+    SRObject *sro = NULL;
+    Accessible *focused;
+
+    focused = srl_get_focused_from_desktop ();
+    if (focused)
+    {
+	srl_last_focus = focused;
+	srl_set_info (focused, &last_info_focus);
+	if (!sro_get_from_acc (focused, SRL_EVENT_NAME_FOCUS, &sro))
+	    sro = NULL;
+    }
+
+    return sro;
+}
